' 10/18/13 - Added watchdog set to 512 ms.
'
' 09/23/13 - Modified program to do moving average for LDR ADC and this worked well.
' Also checked RS485 input and that worked well also.
'
' 09/2/13 - Begin program to display outside temperature on two digit LED
' for the master bedroom.  Display intensity is controlled by an LDR.
' Compiled with 2.0.7.6
'
'
$regfile = "m168def.dat"                                    'using ATMEGA168
$crystal = 8000000                                          'Using internal clock

'Dimension variables
Dim Ch As Byte                                              'check for available data
Dim Char As Byte                                            'character from serial port
Dim Hi As Byte                                              'high digit
Dim Lo As Byte                                              'low digit

Dim Total As Word                                           'used in light level moving average
Dim Light As Word                                           'light level from 200 to 1000
Dim Ondelay As Word                                         'digit on time
Dim Offdelay As Word                                        'digit off time
Dim Lightdiff As Word                                       'difference in light level from minimum


'Configure serial port
Open "com1:" For Binary As #1                               'Used for RS485 input and output to LCD
$baud = 9600

'Configure ADC to measure light level
Config Adc = Single , Prescaler = Auto , Reference = Avcc
Start Adc

Config Watchdog = 512

Config Submode = New

'Subroutine to read light level from LDR utilizing 10 point moving average
'and then calculate on and off delays for LED display
Sub Getlightlevel

   Total = 9 * Light
   Total = Total + Getadc(0)
   Light = Total / 10

   If Light >= 200 Then
      Lightdiff = Light - 200
   Else
      Lightdiff = 0
   End If

   Offdelay = Lightdiff * 3
   If Offdelay > 3000 Then
      Offdelay = 3000
   End If
   Ondelay = 3000 - Offdelay

   'Print Chr(254) ; Chr(128) ; Ondelay ; "   ";

End Sub

'Subroutine to check for new temperature and get data for display
'Data is sent as 3 bytes.  The first byte is always "@" and the next
'2 bytes represent the current temperature from 00 to 99 degrees
'If for some reason there are not 2 bytes after the "@", the WDT should kick in
Sub Checkinput

   Ch = Ischarwaiting()
   If Ch = 1 Then
      Get #1 , Char
      If Char = 64 Then                                     'look for "@"

         Do
            Ch = Ischarwaiting()
         Loop Until Ch = 1
         Get #1 , Char
         Hi = Char - 48

         Do
            Ch = Ischarwaiting()
         Loop Until Ch = 1
         Get #1 , Char
         Lo = Char - 48

      End If
   End If

End Sub

'Configure IO and set up alias's
Lodigit Alias Portc.1
Config Lodigit = Output

Hidigit Alias Portc.2
Config Hidigit = Output

Txenable Alias Portd.2
Config Txenable = Output
Txenable = 0

'Output port to 7 segment display
Config Portb = Output

'Initial setup stuff

'Initialize variables
'Turn off multiplex outputs
Reset Hidigit
Reset Lodigit
'Initialize temperature digits
Hi = 0
Lo = 0

'Initialize light level
Light = Getadc(0)
Getlightlevel

'Initialize Sparkfun LCD
Waitms 500
Print #1 , Chr(254) ; Chr(001);
Print "Starting...";
Waitms 500
Print #1 , Chr(254) ; Chr(001);

Start Watchdog

'Main loop - loops forever
Do

   'update light level and see if there is new input
   Getlightlevel
   Checkinput
   Reset Watchdog

   'First display low digit
   Set Lodigit
   Portb = Lookup(lo , Dta)
   Waitus Ondelay
   Reset Lodigit
   Waitus Offdelay

   'update light level and see if there is new input
   Getlightlevel
   Checkinput

   'Second display high digit
   Set Hidigit
   Portb = Lookup(hi , Dta)
   Waitus Ondelay
   Reset Hidigit
   Waitus Offdelay


Loop

End

'This represents the PortB output for the digits 0 through 9
Dta:

Data 63 , 6 , 91 , 79 , 102 , 109 , 125 , 7 , 127 , 111